/*=========================================================================

  Program:   Visualization Toolkit
  Module:    itkFloatingPointExceptions.h

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME itkFloatingPointExceptions - Deal with floating-point exceptions
// .SECTION Description
// Right now it is really basic and it only provides a function to enable
// floating point exceptions on some compilers.
// Note that Borland C++ has floating-point exceptions by default, not
// Visual studio nor gcc. It is mainly use to optionally enable floating
// point exceptions in the C++ tests.

#ifndef __itkFloatingPointExceptions_h
#define __itkFloatingPointExceptions_h

#include "itkWin32Header.h" // for ITK_EXPORT

class ITK_EXPORT itkFloatingPointExceptions
{
public:
  // Description:
  // Enable floating point exceptions.
  static void Enable();
  
  // Description:
  // Disable floating point exceptions.
  static void Disable();
  
private:
  itkFloatingPointExceptions(); // Not implemented.
  itkFloatingPointExceptions(const itkFloatingPointExceptions&);  // Not implemented.
  void operator=(const itkFloatingPointExceptions&);  // Not implemented.
};

#endif
