/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: itkTimeStampTest.cxx,v $
  Language:  C++

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif
#include <iostream>
#include "itkTimeStamp.h"
#include "itkMultiThreader.h"
#include <float.h>

int itkFloatingPointExceptionTest(int argc, char * argv[])
{
  bool success = true;

  if(argc != 3)
    {
    printf("USAGE:  %s <1=CatchFPE|IgnoreFPE=0> <1=FE_DIVBYZERO,2=FE_INVALID,3=FPE_FLTOVF,4=FPE_FLTUND,5=FPE_FLTRES,6=FPE_FLTSUB,7=FPE_INTDIV,8=FPE_INTOVF>",argv[0]);
    }
  //define a zero denominator that the compiler can not see.
  double force_zero_denom=0.0;
  unsigned char large_value_overflow=255;
  if(argc > 100000) //This is never intended to happen, but it provides a way to confuse the compiler.
    {
    force_zero_denom=1.0;
    large_value_overflow=0;
    }
  int           global_fe_code=0;
  double        s;
  unsigned char i;
  //int ExceptionHandlerFailed=0;
  switch(atoi(argv[2]))
    {
  case 1:
    printf("Before FE_DIVBYZERO\n");
    s = 1.0 / force_zero_denom;  // FE_DIVBYZERO
    //if(global_fe_code != FE_DIVBYZERO ) { ExceptionHandlerFailed=1; }
    printf("After FE_DIVBYZERO %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 2:
    printf("Before FE_INVALID\n");
    s = force_zero_denom / force_zero_denom;  // FE_INVALID
    //if(global_fe_code != FE_INVALID ) { ExceptionHandlerFailed=1; }
    printf("After FE_INVALID %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 3:
    printf("Before FPE_FLTOVF\n");
    s = DBL_MAX*DBL_MAX;  // FPE_FLTOVF
    //if(global_fe_code != FPE_FLTOVF ) { ExceptionHandlerFailed=1; }
    printf("After FPE_FLTOVF %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 4:
    printf("Before FPE_FLTUND\n");
    s = DBL_MIN/DBL_MAX;  // FPE_FLTUND
    //if(global_fe_code != FPE_FLTUND ) { ExceptionHandlerFailed=1; }
    printf("After FPE_FLTUND %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 5:
    printf("Before FPE_FLTRES\n");
    s = DBL_MAX+DBL_MIN;  // FPE_FLTRES
    //if(global_fe_code != FPE_FLTRES ) { ExceptionHandlerFailed=1; }
    printf("After FPE_FLTRES %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 6:
    printf("Before FPE_FLTSUB\n");
    s = 1e1054323;  // FPE_FLTSUB
    //if(global_fe_code != FPE_FLTSUB ) { ExceptionHandlerFailed=1; }
    printf("After FPE_FLTSUB %lf   :  %d  \n",s,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 7:
    printf("Before FPE_INTDIV\n");
    i = 1/static_cast<int>(force_zero_denom);  // FPE_INTDIV
    //if(global_fe_code != FPE_INTDIV ) { ExceptionHandlerFailed=1; }
    printf("After FPE_INTDIV %d   :  %d  \n",i,global_fe_code);
    success = false; //Should not make it this far;
    break;
  case 8:
    printf("Before FPE_INTOVF\n");
    i = large_value_overflow;  // FPE_INTOVF
    printf("Between FPE_INTOVF %d   :  %d  \n",i,global_fe_code);
    i++;     // FPE_INTOVF
    printf("Between FPE_INTOVF %d   :  %d  \n",i,global_fe_code);
    i=large_value_overflow*large_value_overflow;     // FPE_INTOVF
    //if(global_fe_code != FPE_INTOVF ) { ExceptionHandlerFailed=1; }
    printf("After FPE_INTOVF %d   :  %d  \n",i,global_fe_code);
    success = false; //Should not make it this far;
    break;
  default:
    printf("Invalid FPE test chosen\n");
    success = false; //Should not make it this far;
    break;
    }

  if (!success)
    {
    std::cerr << "[TEST FAILED]" << std::endl;
    return EXIT_FAILURE;
    }

  std::cout << "[TEST PASSED]" << std::endl;
  return EXIT_SUCCESS;
}
